/**************************************************************************//**
*   @file   ad7298.c
*   @brief  Implementation of AD7298 Driver for NIOS processor.
*   @author grohmast
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 158
******************************************************************************/
#define AD7298_GLOBALS

/*****************************************************************************/
/* Include Files															 */
/*****************************************************************************/
#include "ad7298.h"
#include <altera_avalon_pio_regs.h>         // PIO access macros.
#include <altera_avalon_spi.h>              // SPI.
#include <unistd.h>                         // usleep() function.
#include <system.h>
#if (USE_AD7298 == 1)

/*****************************************************************************/
/*  Private variables                                                        */
/*****************************************************************************/
unsigned char	txBuffer[4] = {0,};		// Tx SPI Buffer.
unsigned char   rxBuffer[4] = {0,};		// Rx SPI Buffer.

/*****************************************************************************/
/* AD7298                                                                    */
/*****************************************************************************/
extern ad7298_data_t   ad7298_data[9];				// ADC Data
static const int aNumberOfADCChannelsToRead = 9;

/**************************************************************************//**
 * @brief Initialize the ad7298 board and enable SPI.
 *
 * @param  None.
 *
 * @return SUCCESS.
******************************************************************************/
int AD7298_Init(void)
{
	// Enable output buffers of standard SPI interface, invert SPI clock
	SetCtrl(AD7298_SPI_CFG);
	IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x01);   // GPIO0 = output
	IOWR_ALTERA_AVALON_PIO_DATA(GPIO_BASE, 0x01);        // GPIO0 = high
	// Setup all channels in repeat mode.
	AD7298_RepeatChannel( ADCCH0
						| ADCCH1
						| ADCCH2
						| ADCCH3
						| ADCCH4
						| ADCCH5
						| ADCCH6
						| ADCCH7
						// | CHTSENS
						| CHTAVG );	// repeat
	// Set according to channels configured.
	return(SUCCESS);
}

/**************************************************************************//**
 * @brief Set configuration.
 *
 * @param  value - configuration.
 *
 * @return SUCCESS.
******************************************************************************/
int AD7298_Set(int value)
{
	txBuffer[0] = (unsigned char) ((value & 0x00ff00) >> 8);
	txBuffer[1] = (unsigned char)  (value & 0x0000ff);
    alt_avalon_spi_command(AD7298_SPI,  	// base address of SPI module
    					   0,              // slave number
    					   2,              // write_length
    					   txBuffer,        // *write_data
    					   0,              // read_length
    					   rxBuffer,       // *read_data
    					   0);               // flags

    return(SUCCESS);
}

/**************************************************************************//**
 * @brief Read a channel value.
 *
 * @param None.
 *
 * @return The channel data.
******************************************************************************/
int AD7298_GetOneAdValue(void)
{
    int returnval;
    txBuffer[0] = (unsigned char) 0;
    txBuffer[1] = (unsigned char) 0;
    alt_avalon_spi_command(AD7298_SPI,  // base address of SPI module
    					   0,           // slave number
    					   0,           // write_length
    					   txBuffer,    // *write_data
    					   2,           // read_length
    					   rxBuffer,    // *read_data
    					   0);          // flags
    returnval  = ((int) rxBuffer[0]) << 8;
    returnval +=  (int) rxBuffer[1];

    return(returnval);
}

/**************************************************************************//**
 * @brief Configure channel.
 *
 * @param Channel no.
 *
 * @return SUCCESS.
******************************************************************************/
int AD7298_SetChannel(int chan)
{
	char flag;
	int iValue;
	flag = chan == CHTAVG ? 1 : 0;		// check average flag

	iValue = ((1 << 15)     // write
	       |  (0 << 14)     // repeat
/*	       |  (0 << 13)     // ch0
		   |  (0 << 12)     // ch1
		   |  (0 << 11)     // ch2
		   |  (0 << 10)     // ch3
		   |  (0 <<  9)     // ch4
		   |  (0 <<  8)     // ch5
		   |  (0 <<  7)     // ch6
		   |  (0 <<  6)     // ch7
		   |  (1 <<  5)     // tsense */
	       |  (chan << 5)	// channel
		   |  (0 <<  4)     // don't care
		   |  (0 <<  3)     // don't care
		   |  (0 <<  2)     // ext_ref
		   |  (flag <<  1)  // tsense_avg
		   |  (0 <<  0));   // ppd
	AD7298_Set(iValue);
	return (SUCCESS);
}

/**************************************************************************//**
 * @brief Configure the channels in repeat mode.
 *
 * @param chan - channel configuration.
 *
 * @return SUCCESS.
******************************************************************************/
int AD7298_RepeatChannel(int chan)
{
	char flag;
	int value;
	flag = ((chan & CHTAVG) ? 1 : 0);	// check average flag
	value = ((1 << 15)     // write
	       |  (1 << 14)     // repeat
/*	       |  (0 << 13)     // ch0
		   |  (0 << 12)     // ch1
		   |  (0 << 11)     // ch2
		   |  (0 << 10)     // ch3
		   |  (0 <<  9)     // ch4
		   |  (0 <<  8)     // ch5
		   |  (0 <<  7)     // ch6
		   |  (0 <<  6)     // ch7
		   |  (1 <<  5)     // tsense */
	       |  (chan << 5)	// channel
		   |  (0 <<  4)     // don't care
		   |  (0 <<  3)     // don't care
		   |  (0 <<  2)     // ext_ref
		   |  (flag <<  1)  // tsense_avg
		   |  (0 <<  0));   // ppd
	AD7298_Set(value);
	return (SUCCESS);
}

/**************************************************************************//**
 * @brief Calculate the temperature in units fahrenheit.
 *
 * @param temperature value.
 *
 * @return fahrernheit.
******************************************************************************/
unsigned int AD7298_CalcTemp(int value)
{
	ad7298_data_t	celcius;
	ad7298_data_t	fahrenheit;
	if (D_MASK(value,(TAVG | TSENS)))
	{
		celcius.w = 0;
		value &= 0x0fff;
		celcius.c[1] = value >> 2;
		// calculate decimals (base: 0.25)
		if (value > 0)
		{
			celcius.c[0] = (value & 0x03) * 25;
		}
		else
		{
			celcius.c[0] = (-value & 0x03) * 25;
		}
		fahrenheit.c[1] = (celcius.c[1] * 9 / 5) + 32;
	}

	return(fahrenheit.w);
}

/**************************************************************************//**
 * @brief Read all configured channel values.
 *
 * @param number of channels.
 *
 * @return SUCCESS or FAIL.
******************************************************************************/
int AD7298_Get(unsigned char numberofRead)
{
	unsigned char 	chan;
	int 			value;
	int				retValue = FAIL;
	// to get all channels required to more request are necessary
	numberofRead += 2;
	while (numberofRead--)
	{
		value = AD7298_GetOneAdValue();
		chan = ((value&(0x0f<<12))>>12);
		switch (chan)
		{
		case VIN0 :
		case VIN1 :
		case VIN2 :
		case VIN3 :
		case VIN4 :
		case VIN5 :
		case VIN6 :
		case VIN7 :
			ad7298_data[chan].w = (value &0xFFF);
			retValue = SUCCESS;
			break;
		case TSENS :
		case TAVG :
			ad7298_data[8].w = AD7298_CalcTemp(value);
			retValue = SUCCESS;
			break;
		default :
			retValue = FAIL;
			break;
		}
		usleep(4000);
	}

	return(retValue);
}

//****************************************************************************/
/* @brief Update the device's monitored and controlled parameters.
 *
 * @param None.
 *
 * @return SUCCESS.
******************************************************************************/
int AD7298_UpdateParameters(void)
{
	// read all channels
	AD7298_Get(aNumberOfADCChannelsToRead);
	// assign to temperature
	ad7298_temperatur = ad7298_data[8].c[1];

	return(SUCCESS);
}
#endif /* USE_AD7298 */
